import { Dimensions, StyleSheet, Text, View } from 'react-native';
import React, { memo, useCallback, useMemo, useRef } from 'react';
//@ts-ignore
import { Gesture, GestureDetector } from 'react-native-gesture-handler';
import Animated, { runOnJS, useAnimatedStyle, useDerivedValue, useSharedValue
//@ts-ignore
} from 'react-native-reanimated';
import { findValueLessThanOrEqualToNumber, findIndexLessThanOrEqual } from './utilis/helper_functions';
const SCREEN_WIDTH = Dimensions.get('window').width;
const NO_WIDTH_SPACE = '​';
const TextComp = /*#__PURE__*/memo(({
  word,
  i,
  addRange,
  values,
  textColor,
  highlightedTextColor,
  highlightColor,
  lineBreakHeight = 5,
  lineSpace = 2,
  textStyle,
  textBackgroundColor = 'transparent'
}) => {
  const derivedValue = useDerivedValue(() => {
    const map = values.value;
    let bgColor = textBackgroundColor;
    let color = textColor;
    for (let a = 0; a < map.length; a++) {
      const data = values.value[a];
      if (data.start === i) {
        bgColor = highlightColor;
        color = highlightedTextColor;
      }
      if (data.end >= i && data.start <= i) {
        bgColor = highlightColor;
        color = highlightedTextColor;
      }
    }
    return {
      ...textStyle,
      backgroundColor: bgColor,
      color,
      marginBottom: lineSpace
    };
  });
  const animatedStyles = useAnimatedStyle(() => {
    return derivedValue.value;
  });
  const onLayout = useCallback(e => {
    const {
      y
    } = e.nativeEvent.layout;
    addRange(y, e.nativeEvent.layout, word);
  }, []);
  const styles = StyleSheet.create({
    view: {
      width: SCREEN_WIDTH,
      height: lineBreakHeight
    }
  });
  return word === '<br/>' ? /*#__PURE__*/React.createElement(View, {
    onLayout: onLayout,
    style: styles.view
  }) : /*#__PURE__*/React.createElement(Animated.Text, {
    onLayout: onLayout,
    style: [animatedStyles]
  }, /*#__PURE__*/React.createElement(Text, null, word, " "), NO_WIDTH_SPACE);
});
const BreakText = ({
  words,
  addRange,
  values,
  textColor,
  highlightedTextColor,
  highlightColor,
  lineBreakHeight,
  lineSpace,
  textStyle,
  textBackgroundColor
}) => {
  return /*#__PURE__*/React.createElement(React.Fragment, null, words.map((word, i) => {
    return /*#__PURE__*/React.createElement(TextComp, {
      key: i,
      word,
      i,
      addRange,
      values,
      textColor,
      highlightedTextColor,
      highlightColor,
      lineBreakHeight,
      lineSpace,
      textStyle,
      textBackgroundColor
    });
  }));
};
const HighlightText = /*#__PURE__*/React.forwardRef(({
  text,
  textColor,
  highlightedTextColor,
  highlightColor,
  lineBreakHeight,
  lineSpace,
  highlightInitialDelay = 150,
  onHighlightStart,
  onHighlightEnd,
  initialHighlightData = [],
  textStyle,
  marginBottom,
  marginTop,
  marginLeft,
  marginRight,
  onHighlightTapped,
  clearHighlightOnTap,
  margin,
  backgroundColor,
  textBackgroundColor
}, ref) => {
  const words = useMemo(() => text.split(' '), [text]);
  const layout = useSharedValue();
  const rangeMap = useSharedValue({});
  const start = useSharedValue();
  const values = useSharedValue(initialHighlightData);
  const prev = useSharedValue();
  const layoutDone = useRef(false);
  let map = {};
  let count = 0;
  function addRange(lowerBound, value) {
    'worklet';

    if (layoutDone.current) {
      map = {};
      count = 0;
      layoutDone.current = false;
    }
    const arr = map[lowerBound] ?? [];
    arr.push(value.x + value.width);
    map[lowerBound] = arr.sort((a, b) => a - b);
    count++;
    if (count >= words.length) {
      rangeMap.value = map;
      layoutDone.current = true;
    }
  }
  const calTap = e => {
    'worklet';

    const keys = Object.keys(rangeMap.value);
    const {
      result,
      words
    } = findValueLessThanOrEqualToNumber(keys, e.absoluteY - layout.value.y, rangeMap.value);
    const i = findIndexLessThanOrEqual(rangeMap.value[result], e.absoluteX - layout.value.x);
    return {
      i,
      words
    };
  };
  const getHighlightedData = () => {
    return values.value;
  };
  const deleteHighlight = id => {
    values.value.splice(id, 1, {
      start: -1,
      end: -1
    });
    values.value = [...values.value];
  };
  React.useImperativeHandle(ref, () => ({
    getHighlightedData,
    deleteHighlight
  }));
  const tap = Gesture.Tap().onStart(e => {
    const {
      i,
      words
    } = calTap(e);
    for (let a = values.value.length - 1; a >= 0; a--) {
      const item = values.value[a];
      if (i + words >= item.start && i + words <= item.end) {
        if (clearHighlightOnTap) {
          runOnJS(deleteHighlight)(a);
        }
        runOnJS(onHighlightTapped)(a, e);
        break;
      }
    }
  });
  const pan = Gesture.Pan().activateAfterLongPress(highlightInitialDelay).onStart(e => {
    runOnJS(onHighlightStart)();
    const {
      i,
      words
    } = calTap(e);
    start.value = i + words;
    prev.value = values.value;
    values.value = [...prev.value, {
      start: i + words,
      end: i + words
    }];
  }).onUpdate(e => {
    if (e.absoluteY >= layout.value.y && e.absoluteX < layout.value.x + layout.value.width) {
      const {
        i,
        words
      } = calTap(e);
      if (i + words < start.value) {
        values.value = [...values.value.slice(0, values.value.length - 1), {
          start: i + words,
          end: start.value
        }];
      } else {
        values.value = [...values.value.slice(0, values.value.length - 1), {
          start: start.value,
          end: i + words
        }];
      }
    }
  }).onEnd(() => {
    runOnJS(onHighlightEnd)(values.value.length - 1);
  });
  const composed = Gesture.Race(tap, pan);
  const styles = StyleSheet.create({
    mainViewStyle: {
      flexDirection: 'row',
      flexWrap: 'wrap',
      margin,
      marginBottom,
      marginTop,
      marginLeft,
      marginRight,
      backgroundColor
    }
  });
  const onLayout = useCallback(e => {
    layout.value = e.nativeEvent.layout;
  }, [layout]);
  return /*#__PURE__*/React.createElement(GestureDetector, {
    gesture: composed
  }, /*#__PURE__*/React.createElement(View, {
    style: styles.mainViewStyle,
    onLayout: onLayout
  }, /*#__PURE__*/React.createElement(BreakText, {
    words,
    addRange,
    values,
    textColor,
    highlightedTextColor,
    highlightColor,
    lineBreakHeight,
    lineSpace,
    textStyle,
    textBackgroundColor
  })));
});
export default HighlightText;
//# sourceMappingURL=HighlightText.js.map